/****************************************************************************/
/* Copyright (c) 2005-2011, SAS Institute Inc., Cary, NC, USA,              */
/*                       All Rights Reserved.                               */
/*                                                                          */
/* Name:    mo_gui_scenario                                                 */
/*                                                                          */
/****************************************************************************/

%macro mo_gui_scenario;

proc sql noprint;
   create table scenario_parameters_tmp as
      select a.scenario_id, 
             b.customer_id_column_name as customer_id,
             b.household_id_column_name as household_id,
             case
                when a.objective_operator = &_MIN_ then "MIN"
                when a.objective_operator = &_MAX_ then "MAX"
                else ""
             end as objective_operator length=8,
             a.objective_attribute_name as objective_numeric_measure,
             a.active_suppression_rules as active_suppression_rules_flg,
             a.active_constraints as active_constraints_flg,
             b.agent_column_name as agent_keyword length=32,
             a.active_max_min_contact_policies as active_max_min_policies_flg,
             a.active_blocking_contact_policies as active_blocking_policies_flg,
             b.uses_prioritization as use_priorities_flg,
             b.customer_rank_measure_name as priorities_customer_rank_measure length=32,
             b.priority_column_name as priority_comm_column_name length=32,
             b.contact_history_table_name length=32,
             b.communication_history_table_name length=32
      from scenarios as a, input_data as b, projects as d
      where a.project_id = d.project_id and d.input_data_id = b.input_data_id;
quit;
%mo_error_tracking;

proc sql noprint;
   create table scenario_parameters as
      select a.*,
             b.active_roi as active_roi_flg,
             b.roi_numerator_attribute_name as roi_numerator_measure length=32,
             b.roi_denominator_attribute_name as roi_denominator_measure length=32,
             b.active_sample, b.sample_ratio, b.sample_seed,
             b.active_secondary_obj as active_secondary_objective_flg,
             case
                when b.secondary_obj_operator = &_MIN_ then "MIN"
                when b.secondary_obj_operator = &_MAX_ then "MAX"
                else ""
             end as secondary_objective_operator length=8,
             b.secondary_obj_attribute_name as secondary_objective_measure,
             b.secondary_obj_tolerance as secondary_objective_tolerance,
             b.active_secondary_obj_range as active_dual_analysis_flg,
             b.secondary_obj_range as dual_analysis_range,
             1 as report_level,
             b.active_sensitivity, b.sensitivity_range, 
             b.active_core_solution as solution_required_cols_only_flg, 
             b.active_agent_constraints as active_agent_constraints_flg,
             b.active_contact_history as active_contact_history_flg
      from scenario_parameters_tmp as a, scenario_advanced_options as b
      where a.scenario_id=b.scenario_id;
quit;
%mo_error_tracking;

data scenario_parameters (drop = scenario_id active_sample active_sensitivity
                                 priority_comm_column_name);
   set scenario_parameters;
   
   if active_sample = 0 then sample_ratio = 1;
   else sample_ratio = sample_ratio/100;
   
   if active_sensitivity = 1 then report_level = 2;
   sensitivity_range = sensitivity_range/100;
   
   secondary_objective_tolerance = secondary_objective_tolerance/100;
   dual_analysis_range = dual_analysis_range/100;
   
   if use_priorities_flg = 1 then do;
      objective_operator = "MIN";
      objective_numeric_measure = priority_comm_column_name;
   end;
run;
%mo_error_tracking;


/* Convert offer filters into communication_filter_txt expressions. */
proc sql noprint;
   create table _offer_filters_1 as
   select a.offer_filter_row_id,
          a.scenario_item_type,
          a.scenario_item_id,
          a.row_order,
          case
             when a.logical_operator = &_LOGICAL_OPERATOR_AND_ then 'and'
             when a.logical_operator = &_LOGICAL_OPERATOR_OR_ then 'or'
             else '' 
          end as logical_operator length=8,
          a.comm_attribute_type,
          a.comm_attribute_id,
          case
             when a.operator = &_FILTER_EQ_ then '='
             when a.operator = &_FILTER_LT_ then '<'
             when a.operator = &_FILTER_LE_ then '<='
             when a.operator = &_FILTER_GT_ then '>'
             when a.operator = &_FILTER_GE_ then '>='
             when a.operator = &_FILTER_NE_ then '~='
             when a.operator = &_FILTER_BETWEEN_ then 'between'
             when a.operator = &_FILTER_IN_ then 'in'
             when a.operator = &_FILTER_NOT_IN_ then 'not in'
             else ''
          end as operator length=8,
          a.filter_value,
          a.filter_value2,
          b.comm_attribute_distinct_value_id
   from offer_filters as a left join offer_filter_attribute_values as b
   on a.offer_filter_row_id=b.offer_filter_row_id;
quit;
%mo_error_tracking;

proc sql noprint;
   create table _offer_filters_2 as
      select a.*, b.comm_attribute_name as comm_char_attribute_name
      from _offer_filters_1 as a left join comm_character_attributes as b
      on a.comm_attribute_id = b.comm_char_attribute_id
      and comm_attribute_type = &_COMM_ATTRIBUTE_TYPE_CHAR_;
quit;
%mo_error_tracking;

proc sql noprint;
   create table _offer_filters_3 as
      select a.*, b.comm_attribute_name as comm_num_attribute_name
      from _offer_filters_2 as a left join comm_numeric_attributes as b
      on a.comm_attribute_id = b.comm_attribute_id
      and comm_attribute_type = &_COMM_ATTRIBUTE_TYPE_NUM_;
quit;
%mo_error_tracking;

proc sql noprint;
   create table _offer_filters_4 as
      select a.*, b.attribute_name as comm_calc_attribute_name
      from _offer_filters_3 as a left join calculated_attributes as b
      on a.comm_attribute_id = b.calculated_attribute_id
      and comm_attribute_type = &_COMM_ATTRIBUTE_TYPE_CALC_;
quit;
%mo_error_tracking;

proc sql noprint;
   create table _offer_filters_5 as
      select a.*, b.attribute_name as comm_filt_attribute_name
      from _offer_filters_4 as a left join offer_filter_measures as b
      on a.comm_attribute_id = b.offer_filter_measure_id
      and comm_attribute_type = &_COMM_ATTRIBUTE_TYPE_FILTER_;
quit;
%mo_error_tracking;

proc sql noprint;
   create table _offer_filters_6 as
      select a.*, b.comm_attribute_value
      from _offer_filters_5 as a left join comm_character_distinct_values as b
      on a.comm_attribute_distinct_value_id = b.comm_attribute_distinct_value_id
      order by scenario_item_type, scenario_item_id, row_order;
quit;
%mo_error_tracking;

data _offer_filters_7;
   set _offer_filters_6;
   by scenario_item_type scenario_item_id row_order;
   
   if comm_attribute_type = &_COMM_ATTRIBUTE_TYPE_CHAR_ then len1 = length(quote(trim(comm_attribute_value))) + 2;
   else len1 = 32;

   if last.row_order then do;
      if comm_attribute_type = &_COMM_ATTRIBUTE_TYPE_CHAR_ then len2 = length(comm_char_attribute_name);
      else if comm_attribute_type = &_COMM_ATTRIBUTE_TYPE_NUM_ then len2 = length(comm_num_attribute_name);
      else if comm_attribute_type = &_COMM_ATTRIBUTE_TYPE_CALC_ then len2 = length(comm_calc_attribute_name);
      else if comm_attribute_type = &_COMM_ATTRIBUTE_TYPE_FILTER_ then len2 = length(comm_filt_attribute_name); 

      len2 = len2 + 2;
      len3 = length(operator) + 4;
      len4 = length(logical_operator) + 2;
   end;
   else do;
      len2 = 0;
      len3 = 0;
      len4 = 0;
   end;
   total_len = sum(len1, len2, len3, len4);
run;
%mo_error_tracking;
   
proc sql noprint;
   create table _offer_filters_8 as
      select scenario_item_type, scenario_item_id, sum(total_len) as length
      from _offer_filters_7
      group by scenario_item_type, scenario_item_id;
   select max(length) into :create_commfilter_maxlen from _offer_filters_8;
quit;
%mo_error_tracking;
%if &create_commfilter_maxlen = . %then %let create_commfilter_maxlen = &_LENGTH_1024_;
%let create_commfilter_maxlen = %sysfunc(max(&create_commfilter_maxlen, &_LENGTH_1024_));

data _offer_filters_txt (keep = scenario_item_type scenario_item_id communication_filter_txt);
   format communication_filter_txt $&create_commfilter_maxlen..;
   retain communication_filter_txt;
   set _offer_filters_6;
   by scenario_item_type scenario_item_id row_order;
   if first.scenario_item_id then communication_filter_txt = '';

   if comm_attribute_type = &_COMM_ATTRIBUTE_TYPE_CHAR_ then comm_attribute_name = comm_char_attribute_name;
   else if comm_attribute_type = &_COMM_ATTRIBUTE_TYPE_NUM_ then comm_attribute_name = comm_num_attribute_name;
   else if comm_attribute_type = &_COMM_ATTRIBUTE_TYPE_CALC_ then comm_attribute_name = comm_calc_attribute_name;
   else if comm_attribute_type = &_COMM_ATTRIBUTE_TYPE_FILTER_ then comm_attribute_name = comm_filt_attribute_name; 

   if comm_attribute_type in (&_COMM_ATTRIBUTE_TYPE_NUM_,&_COMM_ATTRIBUTE_TYPE_CALC_,&_COMM_ATTRIBUTE_TYPE_FILTER_) then do;
      if operator ne 'between' then communication_filter_txt = trim(left(communication_filter_txt)) || ' ' || trim(left(comm_attribute_name)) || ' ' || trim(left(operator)) || ' ' || trim(left(filter_value));
      else communication_filter_txt = trim(left(communication_filter_txt)) || ' ' || trim(left(filter_value)) || ' <= ' || trim(left(comm_attribute_name)) || ' <= ' || trim(left(filter_value2));
   end;
   else do;
      if operator in ('in','not in') then do;
         if first.row_order then communication_filter_txt = trim(left(communication_filter_txt)) || ' ' || trim(left(comm_attribute_name)) || ' ' || trim(left(operator)) || ' (' || quote(trim(comm_attribute_value));
         else communication_filter_txt = trim(left(communication_filter_txt)) || ',' || quote(trim(comm_attribute_value));

         if last.row_order then communication_filter_txt = trim(left(communication_filter_txt)) || ')';
      end;
      else communication_filter_txt = trim(left(communication_filter_txt)) || ' ' || trim(left(comm_attribute_name)) || ' ' || trim(left(operator)) || ' ' || quote(trim(filter_value));
   end;
   if logical_operator ne '' and last.row_order then communication_filter_txt = trim(left(communication_filter_txt)) || ' ' || trim(left(logical_operator)) || ' ';
   if last.scenario_item_id then do;
      communication_filter_txt = trim(left(communication_filter_txt));
      output;
   end;
run;
%mo_error_tracking;

/* Convert customer filters into customer_filter_txt expressions. */
proc sql noprint;
   create table _customer_filters_1 as
   select a.customer_filter_row_id,
          a.scenario_item_type,
          a.scenario_item_id,
          a.row_order,
          case
             when a.logical_operator = &_LOGICAL_OPERATOR_AND_ then 'and'
             when a.logical_operator = &_LOGICAL_OPERATOR_OR_ then 'or'
             else '' 
          end as logical_operator length=8,
          a.customer_attribute_type,
          a.customer_attribute_id,
          case
             when a.operator = &_FILTER_EQ_ then '='
             when a.operator = &_FILTER_LT_ then '<'
             when a.operator = &_FILTER_LE_ then '<='
             when a.operator = &_FILTER_GT_ then '>'
             when a.operator = &_FILTER_GE_ then '>='
             when a.operator = &_FILTER_NE_ then '~='
             when a.operator = &_FILTER_BETWEEN_ then 'between'
             when a.operator = &_FILTER_IN_ then 'in'
             when a.operator = &_FILTER_NOT_IN_ then 'not in'
             else ''
          end as operator length=8,
          a.filter_value,
          a.filter_value2,
          b.customer_attribute_value_id
   from customer_filters as a left join customer_filter_attribute_values as b
   on a.customer_filter_row_id=b.customer_filter_row_id;
quit;
%mo_error_tracking;

proc sql noprint;
   create table _customer_filters_2 as
      select a.*, b.customer_attribute as customer_attribute_indata
      from _customer_filters_1 as a left join customer_attributes as b
      on a.customer_attribute_id = b.customer_attribute_id
      and customer_attribute_type in (&_CUST_ATTRIBUTE_TYPE_NUM_,&_CUST_ATTRIBUTE_TYPE_CHAR_);
quit;
%mo_error_tracking;

proc sql noprint;
   create table _customer_filters_3 as
      select a.*, b.customer_offer_attribute as customer_attribute_custcomm
      from _customer_filters_2 as a left join customer_offer_attributes as b
      on a.customer_attribute_id = b.customer_offer_attribute_id
      and customer_attribute_type = &_CUST_ATTRIBUTE_TYPE_CUSTCOMM_;
quit;
%mo_error_tracking;

proc sql noprint;
   create table _customer_filters_4 as
      select a.*, b.attribute_name as customer_attribute_calc, b.attribute_type as calc_attribute_type
      from _customer_filters_3 as a left join calculated_attributes as b
      on a.customer_attribute_id = b.calculated_attribute_id
      and customer_attribute_type = &_CUST_ATTRIBUTE_TYPE_CALC_;
quit;
%mo_error_tracking;

proc sql noprint;
   create table _customer_filters_5 as
      select a.*, b.attribute_name as customer_attribute_filter
      from _customer_filters_4 as a left join customer_filter_measures as b
      on a.customer_attribute_id = b.customer_filter_measure_id
      and customer_attribute_type = &_CUST_ATTRIBUTE_TYPE_FILTER_;
quit;
%mo_error_tracking;

proc sql noprint;
   create table _customer_filters_6 as
      select a.*, b.customer_attribute_value
      from _customer_filters_5 as a left join customer_attribute_values as b
      on a.customer_attribute_value_id = b.customer_attribute_value_id
      order by scenario_item_type, scenario_item_id, row_order;
quit;
%mo_error_tracking;

data _customer_filters_7;
   set _customer_filters_6;
   by scenario_item_type scenario_item_id row_order;
   
   if customer_attribute_type = &_CUST_ATTRIBUTE_TYPE_CHAR_ then len1 = length(quote(trim(customer_attribute_value))) + 2;
   else len1 = 32;
   
   if last.row_order then do;
      if customer_attribute_type in (&_CUST_ATTRIBUTE_TYPE_NUM_,&_CUST_ATTRIBUTE_TYPE_CHAR_) 
         then len2 = length(customer_attribute_indata);
      else if customer_attribute_type = &_CUST_ATTRIBUTE_TYPE_CUSTCOMM_ 
         then len2 = length(customer_attribute_custcomm);
      else if customer_attribute_type = &_CUST_ATTRIBUTE_TYPE_CALC_ 
         then len2 = length(customer_attribute_calc);
      else if customer_attribute_type = &_CUST_ATTRIBUTE_TYPE_FILTER_ 
         then len2 = length(customer_attribute_filter);

      len2 = len2 + 2;
      len3 = length(operator) + 4;
      len4 = length(logical_operator) + 2;
   end;
   else do;
      len2 = 0;
      len3 = 0;
      len4 = 0;
   end;
   total_len = sum(len1, len2, len3, len4);
run;
%mo_error_tracking;
   
proc sql noprint;
   create table _customer_filters_8 as
      select scenario_item_type, scenario_item_id, sum(total_len) as length
      from _customer_filters_7
      group by scenario_item_type, scenario_item_id;
   select max(length) into :create_custfilter_maxlen from _customer_filters_8;
quit;
%mo_error_tracking;
%if &create_custfilter_maxlen = . %then %let create_custfilter_maxlen = &_LENGTH_1024_;
%let create_custfilter_maxlen = %sysfunc(max(&create_custfilter_maxlen, &_LENGTH_1024_));

data _customer_filters_txt (keep = scenario_item_type scenario_item_id customer_filter_txt);
   format customer_filter_txt $&create_custfilter_maxlen..;
   retain customer_filter_txt;
   set _customer_filters_6;
   by scenario_item_type scenario_item_id row_order;
   if first.scenario_item_id then customer_filter_txt = '';

   if customer_attribute_type in (&_CUST_ATTRIBUTE_TYPE_NUM_,&_CUST_ATTRIBUTE_TYPE_CHAR_) 
      then customer_attribute_name = customer_attribute_indata;
   else if customer_attribute_type = &_CUST_ATTRIBUTE_TYPE_CUSTCOMM_ 
      then customer_attribute_name = customer_attribute_custcomm;
   else if customer_attribute_type = &_CUST_ATTRIBUTE_TYPE_CALC_ 
      then customer_attribute_name = customer_attribute_calc;
   else if customer_attribute_type = &_CUST_ATTRIBUTE_TYPE_FILTER_ 
      then customer_attribute_name = customer_attribute_filter;

   if (customer_attribute_type in (&_CUST_ATTRIBUTE_TYPE_NUM_, &_CUST_ATTRIBUTE_TYPE_FILTER_)) or
      (customer_attribute_type = &_CUST_ATTRIBUTE_TYPE_CALC_ and calc_attribute_type = &_ATTRIBUTE_TYPE_CUST_) then do;
      if operator ne 'between' then customer_filter_txt = trim(left(customer_filter_txt)) || ' ' || trim(left(customer_attribute_name)) || ' ' || trim(left(operator)) || ' ' || trim(left(filter_value));
      else customer_filter_txt = trim(left(customer_filter_txt)) || ' ' || trim(left(filter_value)) || ' <= ' || trim(left(customer_attribute_name)) || ' <= ' || trim(left(filter_value2));
   end;
   else if (customer_attribute_type = &_CUST_ATTRIBUTE_TYPE_CUSTCOMM_) or
      (customer_attribute_type = &_CUST_ATTRIBUTE_TYPE_CALC_ and calc_attribute_type = &_ATTRIBUTE_TYPE_CUST_COMM_) then do;
      if operator ne 'between' then customer_filter_txt = trim(left(customer_filter_txt)) || ' [' || trim(left(customer_attribute_name)) || '] ' || trim(left(operator)) || ' ' || trim(left(filter_value));
      else customer_filter_txt = trim(left(customer_filter_txt)) || ' ' || trim(left(filter_value)) || ' <= [' || trim(left(customer_attribute_name)) || '] <= ' || trim(left(filter_value2));
   end;
   else do;
      if operator in ('in','not in') then do;
         if first.row_order then customer_filter_txt = trim(left(customer_filter_txt)) || ' ' || trim(left(customer_attribute_name)) || ' ' || trim(left(operator)) || ' (' || quote(trim(customer_attribute_value));
         else customer_filter_txt = trim(left(customer_filter_txt)) || ',' || quote(trim(customer_attribute_value));

         if last.row_order then customer_filter_txt = trim(left(customer_filter_txt)) || ')';
      end;
      else customer_filter_txt = trim(left(customer_filter_txt)) || ' ' || trim(left(customer_attribute_name)) || ' ' || trim(left(operator)) || ' ' || quote(trim(filter_value));
   end;
   if logical_operator ne '' and last.row_order then customer_filter_txt = trim(left(customer_filter_txt)) || ' ' || trim(left(logical_operator)) || ' ';
   if last.scenario_item_id then do;
      customer_filter_txt = trim(left(customer_filter_txt));
      output;
   end;
run;
%mo_error_tracking;

proc sql noprint;
   select active_agent_constraints_flg, agent_keyword
   into :has_agent_constraints, :agent_keyword
   from scenario_parameters;
quit;
%mo_error_tracking;

%if &has_agent_constraints = 1 %then %do;
   proc sql noprint;
      create table agent_capacity as
         select a.agent as &agent_keyword, b.time_period, a.capacity
         from agents as a left join time_periods as b
         on a.time_period_id = b.time_period_id;
   quit;
   %mo_error_tracking;
%end;

%mend;
